@@ -188,7 +188,7 @@ static bool GetUnicodeInfo(Cursor* cur, Py_ssize_t index, PyObject* param, Param
 
     if (len <= cur->cnxn->wvarchar_maxlength)
     {
-        if (SQLWCHAR_SIZE == Py_UNICODE_SIZE)
+        if (ODBCCHAR_SIZE == Py_UNICODE_SIZE)
         {
             info.ParameterValuePtr = pch;
         }
@@ -209,14 +209,14 @@ static bool GetUnicodeInfo(Cursor* cur, Py_ssize_t index, PyObject* param, Param
         }
 
         info.ParameterType = SQL_WVARCHAR;
-        info.StrLen_or_Ind = (SQLINTEGER)(len * sizeof(SQLWCHAR));
+        info.StrLen_or_Ind = (SQLINTEGER)(len * ODBCCHAR_SIZE);
     }
     else
     {
         // Too long to pass all at once, so we'll provide the data at execute.
 
         info.ParameterType     = SQL_WLONGVARCHAR;
-        info.StrLen_or_Ind     = cur->cnxn->need_long_data_len ? SQL_LEN_DATA_AT_EXEC((SQLLEN)len * sizeof(SQLWCHAR)) : SQL_DATA_AT_EXEC;
+        info.StrLen_or_Ind     = cur->cnxn->need_long_data_len ? SQL_LEN_DATA_AT_EXEC((SQLLEN)len * ODBCCHAR_SIZE) : SQL_DATA_AT_EXEC;
         info.ParameterValuePtr = param;
     }
 
@@ -303,6 +303,7 @@ static bool GetIntInfo(Cursor* cur, Py_ssize_t index, PyObject* param, ParamInfo
 
 #if LONG_BIT == 64
     info.ValueType     = SQL_C_SBIGINT;
+    // info.ValueType     = SQL_C_LONG;
     info.ParameterType = SQL_BIGINT;
 #elif LONG_BIT == 32
     info.ValueType     = SQL_C_LONG;
@@ -341,6 +342,8 @@ static bool GetFloatInfo(Cursor* cur, Py_ssize_t index, PyObject* param, ParamIn
 
 static char* CreateDecimalString(long sign, PyObject* digits, long exp)
 {
+    // Allocate an ASCII string containing the given decimal.
+
     long count = (long)PyTuple_GET_SIZE(digits);
 
     char* pch;
@@ -683,7 +686,7 @@ bool PrepareAndBind(Cursor* cur, PyObject* pSql, PyObject* original_params, bool
         {
             SQLWChar sql(pSql);
             Py_BEGIN_ALLOW_THREADS
-            ret = SQLPrepareW(cur->hstmt, sql, SQL_NTS);
+            ret = SQLPrepareW(cur->hstmt, sql.get(), SQL_NTS);
             if (SQL_SUCCEEDED(ret))
             {
                 szErrorFunc = "SQLNumParams";
