@@ -180,11 +180,16 @@ static bool import_types()
     if (!Params_init())
         return false;
 
-    PyObject* decimalmod = PyImport_ImportModule("decimal");
+    PyObject* decimalmod = PyImport_ImportModule("cdecimal");
     if (!decimalmod)
     {
-        PyErr_SetString(PyExc_RuntimeError, "Unable to import decimal");
-        return false;
+        // Clear the error from the failed import of cdecimal.
+        PyErr_Clear();
+        decimalmod = PyImport_ImportModule("decimal");
+        if (!decimalmod) {
+            PyErr_SetString(PyExc_RuntimeError, "Unable to import cdecimal or decimal");
+            return false;
+        }
     }
 
     decimal_type = PyObject_GetAttrString(decimalmod, "Decimal");
@@ -306,11 +311,13 @@ static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
                 fAnsi = PyObject_IsTrue(value);
                 continue;
             }
+#if PY_MAJOR_VERSION < 3
             if (Text_EqualsI(key, "unicode_results"))
             {
                 fUnicodeResults = PyObject_IsTrue(value);
                 continue;
             }
+#endif
             if (Text_EqualsI(key, "timeout"))
             {
                 timeout = PyInt_AsLong(value);
